using System;
using System.Collections;
using Server;
using Server.Targeting;

namespace Server.Scripts.Commands
{
	public class Wipe
	{
		public enum WipeType { All, Items, NPCs }

		public static void Initialize()
		{
			Server.Commands.Register( "Wipe", AccessLevel.GameMaster, new CommandEventHandler( WipeAll_OnCommand ) );
			Server.Commands.Register( "WipeItems", AccessLevel.GameMaster, new CommandEventHandler( WipeItems_OnCommand ) );
			Server.Commands.Register( "WipeNPCs", AccessLevel.GameMaster, new CommandEventHandler( WipeNPCs_OnCommand ) );
		}

		private static void WipeAll_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new WipeTarget( WipeType.All );
			e.Mobile.SendMessage( "Target bounding points." );
		}

		private static void WipeItems_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new WipeTarget( WipeType.Items );
			e.Mobile.SendMessage( "Target bounding points." );
		}

		private static void WipeNPCs_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new WipeTarget( WipeType.NPCs );
			e.Mobile.SendMessage( "Target bounding points." );
		}

		public class WipeTarget : Target
		{
			private WipeType m_Type;

			public WipeTarget( WipeType type ) : base( -1, true, TargetFlags.None )
			{
				m_Type = type;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				IPoint3D p = o as IPoint3D;

				if ( p != null )
				{
					if ( p is Item )
					{
						p = ((Item)p).GetWorldLocation();
					}

					from.Target = new InternalTarget( m_Type, new Point3D( p ) );
				}
			}

			private class InternalTarget : Target
			{
				private WipeType m_Type;
				private Point3D m_Point;

				public InternalTarget( WipeType type, Point3D p ) : base( -1, true, TargetFlags.None )
				{
					m_Type = type;
					m_Point = p;
				}

				protected override void OnTarget( Mobile from, object targeted )
				{
					IPoint3D p = targeted as IPoint3D;

					if ( p != null )
					{
						if ( p is Item )
						{
							p = ((Item)p).GetWorldLocation();
						}

						Map map = from.Map;
						Point3D pt = new Point3D( p );
						Utility.FixPoints( ref m_Point, ref pt );
						Rectangle2D rect = new Rectangle2D( m_Point, p );
						ArrayList items = new ArrayList();
						ArrayList mobiles = new ArrayList();

						for ( int x = 0; x <= rect.Width; ++x )
						{
							for ( int y = 0; y <= rect.Height; ++y )
							{
								int tileX = rect.Start.X + x;
								int tileY = rect.Start.Y + y;

								Sector sector = map.GetSector( tileX, tileY );

								if ( m_Type == WipeType.All || m_Type == WipeType.Items )
								{
									for ( int i = 0; i < sector.Items.Count; ++i )
									{
										Item item = (Item)sector.Items[i];

										if ( item.X == tileX && item.Y == tileY )
											items.Add( item );
									}
								}

								if ( m_Type == WipeType.All || m_Type == WipeType.Items )
								{
									for ( int i = 0; i < sector.Mobiles.Count; ++i )
									{
										Mobile m = (Mobile)sector.Mobiles[i];

										if ( m.X == tileX && m.Y == tileY && !m.Player )
											mobiles.Add( m );
									}
								}
							}
						}

						for ( int i=0;i<mobiles.Count;i++ )
						{
							Mobile m = (Mobile)mobiles[i];
							m.Delete();
						}

						for ( int i=0;i<items.Count;i++ )
						{
							Item item = (Item)items[i];
							item.Delete();
						}
					}
				}
			}
		}
	}
}
