using System;
using System.Collections;
using Server.Targeting;
using Server.Network;

//
// This is a first simple AI
//
//

namespace Server.Mobiles
{
	public class VendorAI : BaseAI
	{
		private enum VendorState
		{
            VendorWander,	// Act normaly
			VendorInteract,	// Is in interaction with someone
			VendorFlee		// Try to flee
		}

		private VendorState m_iState;

		public VendorAI(BaseCreature m) : base (m)
		{
			m_iState = VendorState.VendorWander;
		}

		public override bool Think()
		{
			if ( m_Mobile.Deleted )
				return false;

			switch (m_iState)
			{
				case VendorState.VendorWander:
					return DoWander();

				case VendorState.VendorInteract:
					return DoInteract();

				case VendorState.VendorFlee:
					return DoFlee();

				default:
					return false;
			}
		}

		public virtual bool DoWander()
		{
			m_Mobile.DebugSay( "I'm fine" );

			if ( m_Mobile.Combatant != null )
			{
				m_Mobile.DebugSay( "{0} is attacking me", m_Mobile.Combatant.Name );

				m_Mobile.Say( Utility.RandomList( 1005305, 501603 ) );

				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = VendorState.VendorFlee;
			}
			else
			{
				if ( m_Mobile.FocusMob != null )
				{
					m_Mobile.DebugSay( "{0} has talked to me", m_Mobile.FocusMob.Name );

					m_iState = VendorState.VendorInteract;
				}
				else
				{
					m_Mobile.Warmode = false;

					DoWalkRandomInHome( 5, 2, 1 );
				}
			}

			return true;
		}

		public virtual bool DoInteract()
		{
			Mobile customer = m_Mobile.FocusMob;

			if ( m_Mobile.Combatant != null )
			{
				m_Mobile.DebugSay( "{0} is attacking me", m_Mobile.Combatant.Name );

				m_Mobile.Say( Utility.RandomList( 1005305, 501603 ) );

				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = VendorState.VendorFlee;
				
				return true;
			}

			if ( customer == null || customer.Deleted || customer.Map != m_Mobile.Map )
			{
				m_Mobile.DebugSay( "My customer have disapeared" );
				m_Mobile.FocusMob = null;

				m_iState = VendorState.VendorWander;
			}
			else
			{
				if ( customer.InRange( m_Mobile, m_Mobile.RangeFight ) )
				{
					m_Mobile.DebugSay( "I am with {0}", customer.Name );

					m_Mobile.Direction = m_Mobile.GetDirectionTo( customer );
				}
				else
				{
					m_Mobile.DebugSay( "{0} is gone", customer.Name );
					m_Mobile.FocusMob = null;

					m_iState = VendorState.VendorWander;	
				}
			}

			return true;
		}

		public virtual bool DoFlee()
		{
			Mobile combatant = m_Mobile.Combatant;

			if ( combatant == null || combatant.Deleted || combatant.Map != m_Mobile.Map )
			{
				m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
				m_iState = VendorState.VendorWander;	
			}
			else if ( DoWalkMobileRange( combatant, 1, true, m_Mobile.RangePerception, m_Mobile.RangePerception ) )
			{
				m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
				m_iState = VendorState.VendorWander;	
			}
			else
			{
				m_Mobile.DebugSay( "I am afraid of {0}", combatant.Name );
			}

			return true;
		}

		// Temporary
		public override void OnSpeech( SpeechEventArgs e )
		{
			base.OnSpeech( e );

			if ( m_Mobile.FocusMob == null )
			{
				if ( e.Speech == "buy" )
				{
					m_Mobile.FocusMob = e.Mobile;
				}
			}
			else if ( m_Mobile.FocusMob == e.Mobile )
			{
				if ( e.Speech == "bye" )
				{
					m_Mobile.FocusMob = null;                                    
				}
			}
			else
			{
				m_Mobile.DebugSay( "I'm with someone now" );                
			}
		}
	}
}
