using System;
using Server.Items;
using Server.Mobiles;
using Server.Spells;

namespace Server.Mobiles
{
	/// <summary>
	/// This is a test creature
	/// You can set its value in game
	/// It die after 5 minutes, so your test server stay clean
	/// Create a macro to help your creation "[add Dummy 1 15 7 -1 0.5 2"
	/// 
	/// A iTeam of negative will set a faction at random
	/// 
	/// Say Kill if you want them to die
	/// 
	/// </summary>
	public class Dummy : BaseCreature
	{
		public Timer	m_Timer;

		[Constructable]
		public Dummy(AIType iAI, int iRangePerception, int iRangeFight, int iTeam, double dActiveSpeed, double dPassiveSpeed) : base(iAI, iAI, iRangePerception, iRangeFight, iTeam, dActiveSpeed, dPassiveSpeed)
		{
			this.Body = 400 + Utility.Random(2);
			this.InitStats( 100, 100, 100 );

			this.Skills[SkillName.Archery].Base = 100;
			this.Skills[SkillName.Tactics].Base = 100;
			this.Skills[SkillName.Anatomy].Base = 100;
			this.Skills[SkillName.DetectHidden].Base = 100;
			this.Skills[SkillName.Magery].Base = 100;
			this.Skills[SkillName.Swords].Base = 100;
			this.Skills[SkillName.Meditation].Base = 100;
			this.Skills[SkillName.MagicResist].Base = 100;
			this.Skills[SkillName.EvalInt].Base = 100;

			if (iTeam < 0)
			{
				Team = Utility.Random(-iTeam);
			}

			int iHue = 20 + Team * 40;

			SkullCap cap = new SkullCap();
			cap.Hue = iHue;
			AddItem(cap);

			ShortPants pants = new ShortPants();
			pants.Hue = iHue;
			AddItem(pants);

			LeatherGloves glv = new LeatherGloves();
			glv.Hue = iHue;
			AddItem(glv);

			Container pack = new Backpack();

			pack.Movable = false;
			pack.Newbied = true;

			AddItem( pack );

			m_Timer = new AutokillTimer(this);
			m_Timer.Start();
		}

		public Dummy( Serial serial ) : base( serial )
		{
			m_Timer = new AutokillTimer(this);
			m_Timer.Start();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnSpeech( SpeechEventArgs e )
		{
			base.OnSpeech( e );

			if (e.Mobile.AccessLevel >= AccessLevel.GameMaster)
			{
				if (e.Speech == "kill")
				{
					m_Timer.Stop();
					m_Timer.Delay = TimeSpan.FromSeconds( Utility.Random(1, 5) );
					m_Timer.Start();
					
				}
			}
		}

		public override void OnTeamChange()
		{
			int iHue = 20 + Team * 40;

			Item item = FindItemOnLayer( Layer.Pants );

			if ( item != null )
				item.Hue = iHue;

			item = FindItemOnLayer( Layer.Helm );

			if ( item != null )
				item.Hue = iHue;

			item = FindItemOnLayer( Layer.Gloves );

			if ( item != null )
				item.Hue = iHue;
		}

		private class AutokillTimer : Timer
		{
			private Dummy m_Owner;

			public AutokillTimer( Dummy owner ) : base( TimeSpan.FromMinutes(30.0) )
			{
				m_Owner = owner;
			}

			protected override void OnTick()
			{
				m_Owner.Kill();
				Stop();
			}
		}
	}
}
