using System;
using Server.Items;
using Server.Mobiles;
using Server.Spells;

namespace Server.Mobiles
{
	/// <summary>
	/// This is a test creature
	/// You can set its value in game
	/// It die after 5 minutes, so your test server stay clean
	/// Create a macro to help your creation "[add Dummy 1 15 7 -1 0.5 2"
	/// 
	/// A iTeam of negative will set a faction at random
	/// 
	/// Say Kill if you want them to die
	/// 
	/// </summary>
	public class DummyMeelee : Dummy
	{
		[Constructable]
		public DummyMeelee() : base(AIType.AI_Meelee, 15, 1, 0, 0.2, 0.6)
		{
			this.Name = "Fighter";

			Longsword swd = new Longsword();

			swd.Movable = false;
			swd.Newbied = true;
			swd.Crafter = this;
			swd.Quality = WeaponQuality.Low;

			AddItem( swd );
		}

		public DummyMeelee( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class DummyArcher : Dummy
	{
		[Constructable]
		public DummyArcher() : base(AIType.AI_Archer, 15, 7, 0, 0.2, 0.6)
		{
			this.Name = "Archer";

			Bow bow = new Bow();

			bow.Movable = false;
			bow.Newbied = true;
			bow.Crafter = this;
			bow.Quality = WeaponQuality.Low;
			AddItem( bow );

			Arrow arrows = new Arrow( 20 );
			arrows.Newbied = true;
			AddToBackpack( arrows );

			AddToBackpack( new Gold( 250, 500 ) );
		}

		public DummyArcher( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class DummyMage : Dummy
	{
		[Constructable]
		public DummyMage() : base(AIType.AI_Mage, 15, 1, 0, 0.2, 0.6)
		{
			this.Name = "Mage";

			Spellbook book = new Spellbook();

			book.Movable = false;
			book.Newbied = true;
			AddItem( book );

			AddSpellAttack( typeof(Spells.First.MagicArrowSpell) );
			AddSpellAttack( typeof(Spells.First.WeakenSpell) );
			AddSpellAttack( typeof(Spells.Third.FireballSpell) );
			AddSpellDefense( typeof(Spells.Third.WallOfStoneSpell) );
			AddSpellDefense( typeof(Spells.First.HealSpell) );
		}

		public DummyMage( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class DummyHealer : Dummy
	{
		[Constructable]
		public DummyHealer() : base(AIType.AI_Healer, 15, 1, 0, 0.2, 0.6)
		{
			this.Name = "Healer";
		}

		public DummyHealer( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}

/*			
if (iAI == AIType.AI_Animal)
{
	this.Name = "Animal";
}

if (iAI == AIType.AI_Healer)
{
	this.Name = "Healer";

	Mace swd = new Mace();

	swd.Movable = false;
	swd.Newbied = true;
	swd.Crafter = this;
	swd.Quality = WeaponQuality.Low;
	swd.Hue = iHue;

	AddItem( swd );
}

if (iAI == AIType.AI_Vendor)
{
	this.Name = "Vendor";
}
*/