using System;
using System.Collections;
using Server.Network;

namespace Server.Engines.Help
{
	public enum PageType
	{
		Bug,
		Stuck,
		Account,
		Question,
		Suggestion,
		Other
	}

	public class PageEntry
	{
		private Mobile m_Sender;
		private Mobile m_Handler;
		private DateTime m_Sent;
		private string m_Message;
		private PageType m_Type;
		private Point3D m_PageLocation;
		private Map m_PageMap;

		public Mobile Sender
		{
			get
			{
				return m_Sender;
			}
		}

		public Mobile Handler
		{
			get
			{
				return m_Handler;
			}
			set
			{
				m_Handler = value;
			}
		}

		public DateTime Sent
		{
			get
			{
				return m_Sent;
			}
		}

		public string Message
		{
			get
			{
				return m_Message;
			}
		}

		public PageType Type
		{
			get
			{
				return m_Type;
			}
		}

		public Point3D PageLocation
		{
			get
			{
				return m_PageLocation;
			}
		}

		public Map PageMap
		{
			get
			{
				return m_PageMap;
			}
		}

		public PageEntry( Mobile sender, string message, PageType type )
		{
			m_Sender = sender;
			m_Sent = DateTime.Now;
			m_Message = message;
			m_Type = type;
			m_PageLocation = sender.Location;
			m_PageMap = sender.Map;
		}
	}

	public class PageQueue
	{
		private static ArrayList m_List = new ArrayList();

		public static void Initialize()
		{
			Server.Commands.Register( "Pages", AccessLevel.Counselor, new CommandEventHandler( Pages_OnCommand ) );
		}

		private static void Pages_OnCommand( CommandEventArgs e )
		{
			for ( int i = 0; i < m_List.Count; ++i )
			{
				if ( ((PageEntry)m_List[i]).Handler == e.Mobile )
				{
					e.Mobile.SendGump( new PageEntryGump( e.Mobile, (PageEntry)m_List[i] ) );
					return;
				}
			}

			if ( m_List.Count > 0 )
			{
				e.Mobile.SendGump( new PageQueueGump() );
			}
			else
			{
				e.Mobile.SendMessage( "The page queue is empty." );
			}
		}

		public static bool IsHandling( Mobile check )
		{
			for ( int i = 0; i < m_List.Count; ++i )
			{
				if ( ((PageEntry)m_List[i]).Handler == check )
				{
					return true;
				}
			}

			return false;
		}

		public static bool Contains( Mobile sender )
		{
			for ( int i = 0; i < m_List.Count; ++i )
			{
				if ( ((PageEntry)m_List[i]).Sender == sender )
				{
					return true;
				}
			}

			return false;
		}

		public static void Cancel( Mobile sender )
		{
			for ( int i = 0; i < m_List.Count; ++i )
			{
				if ( ((PageEntry)m_List[i]).Sender == sender )
				{
					m_List.RemoveAt( i );
					return;
				}
			}
		}

		public static void Remove( PageEntry e )
		{
			m_List.Remove( e );
		}

		public static ArrayList List
		{
			get
			{
				return m_List;
			}
		}

		public static void Enqueue( PageEntry entry )
		{
			m_List.Add( entry );

			foreach ( NetState ns in NetState.Instances )
			{
				Mobile m = ns.Mobile;

				if ( m != null && m.AccessLevel >= AccessLevel.Counselor && m.AutoPageNotify && !IsHandling( m ) )
				{
					m.SendMessage( "A new page has been placed in the queue." );
				}
			}
		}

		public static PageEntry Dequeue()
		{
			if ( m_List.Count > 0 )
			{
				PageEntry e = (PageEntry)m_List[0];

				m_List.RemoveAt( 0 );

				return e;
			}
			else
			{
				return null;
			}
		}

		public static PageEntry Peek()
		{
			return m_List.Count > 0 ? (PageEntry)m_List[0] : null;
		}
	}
}