using System;
using System.Collections;
using Server.Guilds;
using Server.Network;
using Server.Targeting;
using Server.Prompts;

namespace Server.Gumps
{
	public class GuildGump : Gump
	{
		private Mobile m_Mobile;
		private Guild m_Guild;

		public GuildGump( Mobile beholder, Guild guild ) : base( 50, 50 )
		{
			m_Mobile = beholder;
			m_Guild = guild;

			AddPage( 0 );
			AddBackground( 0, 0, 320, 240, 5054 );
			AddBackground( 10, 10, 300, 220, 3000 );

			AddLabel( 15, 15, 0, guild.Name );

			AddButton( 15, 40, 0xFA5, 0xFA7, 1, GumpButtonType.Reply, 0 );
			AddLabel( 50, 40, 0, "Member List" );

			if ( guild.Leader == beholder )
			{
				AddButton( 15, 65, 0xFA5, 0xFA7, 2, GumpButtonType.Reply, 0 );
				AddLabel( 50, 65, 0, "Recruit Member" );

				AddButton( 15, 90, 0xFA5, 0xFA7, 3, GumpButtonType.Reply, 0 );
				AddLabel( 50, 90, 0, "Grant Title" );
			}
		}

		public override void OnResponse( NetState sender, RelayInfo info )
		{
			switch ( info.ButtonID )
			{
				case 1: // Member List
				{
					if ( m_Guild.IsMember( m_Mobile ) )
					{
						m_Mobile.CloseGump( typeof( GuildGump ) );
						m_Mobile.CloseGump( typeof( GuildMemberList ) );
						m_Mobile.SendGump( new GuildMemberList( m_Mobile, m_Guild, GuildMemberListType.MemberList ) );
					}

					break;
				}
				case 2: // Recruit Member
				{
					if ( m_Guild.Leader == m_Mobile )
						m_Mobile.Target = new GuildRecruitTarget( m_Mobile, m_Guild );

					break;
				}
				case 3: // Grant Title
				{
					if ( m_Guild.Leader == m_Mobile )
					{
						m_Mobile.CloseGump( typeof( GuildGump ) );
						m_Mobile.CloseGump( typeof( GuildMemberList ) );
						m_Mobile.SendGump( new GuildMemberList( m_Mobile, m_Guild, GuildMemberListType.GrantTitle ) );
					}

					break;
				}
			}
		}
	}

	public enum GuildMemberListType
	{
		MemberList = 0,
		GrantTitle = 1
	}

	public class GuildMemberList : Gump
	{
		private Mobile m_Mobile;
		private Guild m_Guild;
		private GuildMemberListType m_Type;

		public GuildMemberList( Mobile beholder, Guild guild, GuildMemberListType type ) : base( 50, 50 )
		{
			m_Type = type;

			m_Mobile = beholder;
			m_Guild = guild;

			AddPage( 0 );

			AddBackground( 0, 0, 320, 240, 5054 );

			AddImageTiled( 10, 10, 220, 23, 0x52 );
			AddImageTiled( 11, 11, 218, 21, 0xBBC );

			AddPage( 1 );

			ArrayList members = guild.Members;

			int buttonID = type == GuildMemberListType.MemberList ? 0xFA2 : 0xFB7;

			for ( int i = 0; i < members.Count; ++i )
			{
				if ( i != 0 && (i % 8) == 0 )
				{
					AddButton( 210, 13, 0x15E1, 0x15E5, 0, GumpButtonType.Page, (i / 5) + 1 );
					AddLabel( 178, 11, 0, "Next" );

					AddPage( (i / 5) + 1 );

					AddButton( 13, 13, 0x15E3, 0x15E7, 0, GumpButtonType.Page, (i / 5) );
					AddLabel( 33, 11, 0, "Previous" );
				}

				AddImageTiled( 10, 32 + ((i % 8) * 25), 220, 26, 0x52 );
				AddImageTiled( 11, 33 + ((i % 8) * 25), 218, 24, 0xBBC );

				AddLabelCropped( 13, 35 + ((i % 8) * 25), 216, 22, 0, ((Mobile)members[i]).Name );

				if ( beholder == guild.Leader )
				{
					AddButton( 11 + 218 - 31, 34 + ((i % 8) * 25), buttonID, buttonID + 2, i + 1, GumpButtonType.Reply, 0 );
				}
			}
		}

		private class GrantTitlePrompt : Prompt
		{
			private Mobile m_Mobile, m_Target;
			private Guild m_Guild;

			public GrantTitlePrompt( Mobile mobile, Guild guild, Mobile target )
			{
				m_Mobile = mobile;
				m_Guild = guild;
				m_Target = target;
			}

			public override void OnResponse( Mobile from, string text )
			{
				if ( m_Guild.Leader == m_Mobile )
				{
					if ( text.Length > 20 )
						text = text.Substring( 0, 20 );

					m_Target.GuildTitle = text == "none" ? null : text;

					m_Mobile.CloseGump( typeof( GuildGump ) );
					m_Mobile.CloseGump( typeof( GuildMemberList ) );
					m_Mobile.SendGump( new GuildGump( m_Mobile, m_Guild ) );
				}
			}

			public override void OnCancel(Mobile from)
			{
				if ( m_Guild.Leader == m_Mobile )
				{
					m_Mobile.CloseGump( typeof( GuildGump ) );
					m_Mobile.CloseGump( typeof( GuildMemberList ) );
					m_Mobile.SendGump( new GuildGump( m_Mobile, m_Guild ) );
				}
			}
		}

		public override void OnResponse( NetState sender, RelayInfo info )
		{
			if ( m_Guild.Leader == m_Mobile && info.ButtonID >= 1 && info.ButtonID <= m_Guild.Members.Count )
			{
				switch ( m_Type )
				{
					case GuildMemberListType.GrantTitle:
					{
						m_Mobile.Prompt = new GrantTitlePrompt( m_Mobile, m_Guild, (Mobile)m_Guild.Members[info.ButtonID - 1] );
						m_Mobile.SendLocalizedMessage( 1011128 ); // Enter new title ...

						break;
					}
					case GuildMemberListType.MemberList:
					{
						m_Guild.RemoveMember( (Mobile) m_Guild.Members[info.ButtonID - 1] );

						m_Mobile.CloseGump( typeof( GuildGump ) );
						m_Mobile.CloseGump( typeof( GuildMemberList ) );
						m_Mobile.SendGump( new GuildMemberList( m_Mobile, m_Guild, m_Type ) );

						break;
					}
				}
			}
			else if ( m_Guild.IsMember( m_Mobile ) )
			{
				m_Mobile.CloseGump( typeof( GuildGump ) );
				m_Mobile.CloseGump( typeof( GuildMemberList ) );
				m_Mobile.SendGump( new GuildGump( m_Mobile, m_Guild ) );
			}
		}
	}
}