using System;
using Server;
using Server.Network;

namespace Server
{
	public class LightCycle
	{
		private static double m_Level;
		private static bool m_DayTime;

		public const int DayLevel = 0;
		public const int NightLevel = 15;
		public const int DungeonLevel = 25;
		private const double Change = ( NightLevel - DayLevel ) / ( 60 / 2.5 );

		static LightCycle()
		{
			m_Level = DayLevel;
			m_DayTime = true;
		}

		public static void Initialize()
		{
			new LightCycleTimer().Start();
			EventSink.Login += new LoginEventHandler( OnLogin );
		
			Server.Commands.Register( "GlobalLight", AccessLevel.GameMaster, new CommandEventHandler( Light_OnCommand ) );
		}

		private static void Light_OnCommand( CommandEventArgs e )
		{
			if ( e.Length >= 1 )
			{
				m_Level = e.GetInt32( 0 );

				e.Mobile.SendMessage( "Setting global light level: {0}", LightCycle.Level );

				for (int i=0;i<NetState.Instances.Count;i++)
				{
					NetState ns = (NetState)NetState.Instances[i];
					Mobile m = ns.Mobile;

					if ( m != null )
						ns.Send( new GlobalLightLevel( (byte)m.Region.LightLevel( m, Level ) ) );
				}
			}
		}

		public static void OnLogin( LoginEventArgs args )
		{
			Mobile m = args.Mobile;
			if ( m.NetState != null )
				m.NetState.Send( new GlobalLightLevel( (byte)m.Region.LightLevel( m, LightCycle.Level ) ) );
		}

		public static double Level
		{
			get
			{
				return m_Level;
			}
			set
			{
				if ( m_Level != value )
				{
					m_Level = value;
					Map.GlobalLight = (int)value;

					if ( m_Level >= NightLevel || m_Level <= DayLevel )
						m_DayTime = !m_DayTime;
				}
			}
		}

		public static bool DayTime
		{
			get
			{
				return m_DayTime;
			}
		}

		private class LightCycleTimer : Timer
		{
			public LightCycleTimer() : base( TimeSpan.FromSeconds( 0 ), TimeSpan.FromMinutes( 2.5 ) )
			{
				Priority = TimerPriority.FiveSeconds;
			}

			//a full day (sunrise to sunrise) will take about 2 hours
			protected override void OnTick()
			{
				if ( LightCycle.DayTime )
					LightCycle.Level += Change;
				else
					LightCycle.Level -= Change;

				//Console.WriteLine( "Setting global light level: {0}", LightCycle.Level );

				for (int i=0;i<NetState.Instances.Count;i++)
				{
					NetState ns = (NetState)NetState.Instances[i];
					Mobile m = ns.Mobile;

					if ( m != null )
						ns.Send( new GlobalLightLevel( (byte)m.Region.LightLevel( m, LightCycle.Level ) ) );
				}
			}
		}

		public class NightSightTimer : Timer
		{
			private Mobile m_Owner;

			public NightSightTimer( Mobile owner ) : base( TimeSpan.FromMinutes( Utility.Random( 5, 3 ) ) )
			{
				m_Owner = owner;
				Priority = TimerPriority.FiveSeconds;
			}

			protected override void OnTick()
			{
				m_Owner.EndAction( typeof( LightCycle ) );
				m_Owner.LightLevel = 0;
			}
		}
	}
}
