// Combat AI v0.1 by Psyk 
// Based on the Aggressive AI by the RunUO Team 

using System;
using System.Collections;

namespace Server.Mobiles
{
	public abstract class AggressiveAI : Mobile
	{
		private Timer m_Timer;

		public AggressiveAI()
		{
			m_Timer = new AITimer( this );
			m_Timer.Start();
		}

		public AggressiveAI( Serial serial ) : base( serial )
		{
			m_Timer = new AITimer( this );
			m_Timer.Start();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public virtual double GetValueFrom( Mobile m )
		{
			return (m.Skills[SkillName.Tactics].Value + m.Str) / this.GetDistanceToSqrt( m );
		}

		public virtual bool CheckAttack( Mobile m )
		{
			return true;
		}

		public override void OnAfterDelete()
		{ 
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public virtual void AquireCombatant()
		{
			if ( Deleted )
				return;

			Map map = this.Map;

			if ( map != null )
			{
				Mobile comb = null;
				double val = 0.0;

				IPooledEnumerable eable = map.GetMobilesInRange( this.Location, 6 );

				foreach ( Mobile m in eable )
				{
					if ( m != this && CanSee( m ) && CheckAttack( m ) )
					{
						double theirVal = GetValueFrom( m );

						if ( comb == null || theirVal > val )
						{
							comb = m;
							val = theirVal;
						}
					}
				}

				eable.Free();

				this.Combatant = comb;
			}
		}

		private class AITimer : Timer
		{
			private AggressiveAI m_Owner;

			public AITimer( AggressiveAI owner ) : base( TimeSpan.FromSeconds( 1.0 ), TimeSpan.FromSeconds( 1.0 ) )
			{
				this.Priority = TimerPriority.FiftyMS;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Deleted )
				{
					Stop();
					return;
				}

				if ( m_Owner.Combatant == null )
				{
					m_Owner.AquireCombatant();
				}
				else
				{
					Mobile c = m_Owner.Combatant;

					if ( !m_Owner.InRange( c, 1 ) )
						m_Owner.Move( m_Owner.GetDirectionTo( c ) | Direction.Running );
				}

				if ( m_Owner.Combatant == null )
				{
					this.Interval = TimeSpan.FromSeconds( 5.0 );
					this.Priority = TimerPriority.OneSecond;
				}
				else
				{
					this.Interval = TimeSpan.FromSeconds( 0.2 );
					this.Priority = TimerPriority.TwentyFiveMS;
				}
			}
		}
	}
}