using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Misc;
using Server.Multis.Deeds;

namespace Server.Mobiles
{
	public class Architect : BaseVendor
	{
		private IShopSellInfo[] m_ArchitectSellInfo = new IShopSellInfo[]{ new ArchitectSellInfo() };
		private IBuyItemInfo[] m_ArchitectBuyInfo = new IBuyItemInfo[]
		{
			//--------------------------Nanme--------------------------------------------Type------------------------------------Price------Amount--------------ID---Hue
			new GenericBuyInfo( "deed to a stone-and-plaster house",			typeof( StonePlasterHouseDeed ),				43800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a field stone house",					typeof( FieldStoneHouseDeed ),					43800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a small brick house",					typeof( SmallBrickHouseDeed),					43800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a wooden house",						typeof( WoodHouseDeed ),						43800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a wood-and-plaster house",				typeof( WoodPlasterHouseDeed ),					43800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a thatched-roof cottage",				typeof( ThatchedRoofCottageDeed ),				43800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a brick house",						typeof( BrickHouseDeed ),						144500,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a two-story wood-and-plaster house",	typeof( TwoStoryWoodPlasterHouseDeed ),			192400,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a tower",								typeof( TowerDeed ),							433200,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "a small stone keep deed",						typeof( KeepDeed ),								665200,	 Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "a castle deed",								typeof( CastleDeed ),							1022800, Utility.Random( 5, 10 ), 0x14F0, 0 ),
			new GenericBuyInfo( "deed to a large house with patio",				typeof( LargePatioDeed ),						152800,  Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a marble house with patio",			typeof( LargeMarbleDeed ),						192000,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a small stone tower",					typeof( SmallTowerDeed ),						88500,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a two story log cabin",				typeof( LogCabinDeed ),							97800,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a sandstone house with patio",			typeof( SandstonePatioDeed ),					90900,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a two story villa",					typeof( VillaDeed ),							136500,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a small stone workshop",				typeof( StoneWorkshopDeed ),					60600,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
			new GenericBuyInfo( "deed to a small marble workshop",				typeof( MarbleWorkshopDeed ),					63000,	 Utility.Random( 5, 10 ), 0x14F0, 0 ), 
		};

		[Constructable()]
		public Architect() : base( "the architect" )
		{
			InitStats( 100, 100, 25 );

			SpeechHue = Utility.RandomDyedHue();
			Hue = Utility.RandomSkinHue();

			if ( this.Female = Utility.RandomBool() )
			{
				this.Body = 0x191;
				this.Name = NameList.RandomFemaleName();
			}
			else
			{
				this.Body = 0x190;
				this.Name = NameList.RandomMaleName();
			}

			AddItem( new FancyShirt( Utility.RandomNeutralHue() ) );
			AddItem( new ShortPants( Utility.RandomNeutralHue() ) );
			AddItem( new Boots( Utility.RandomNeutralHue() ) );

			Item hair = new Item( Utility.RandomList( 0x203B, 0x2049, 0x2048, 0x204A ) );

			hair.Hue = Utility.RandomNondyedHue();
			hair.Layer = Layer.Hair;
			hair.Movable = false;
			hair.Newbied = true;

			AddItem( hair );

			Container pack = new Backpack();

			pack.DropItem( new Gold( 1000, 2000 ) );

			pack.Movable = false;
			pack.Newbied = true;

			AddItem( pack );
		}

		public Architect( Serial serial ) : base( serial )
		{
		}

		public override IShopSellInfo[] GetSellInfo()
		{
			return m_ArchitectSellInfo;
		}

		public override IBuyItemInfo[] GetBuyInfo()
		{
			return m_ArchitectBuyInfo;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class ArchitectSellInfo : IShopSellInfo
	{
		private static Type[] m_MyTypes = new Type[]
		{
			typeof( StonePlasterHouseDeed ),
			typeof( FieldStoneHouseDeed ),
			typeof( SmallBrickHouseDeed ),
			typeof( WoodHouseDeed ),
			typeof( WoodPlasterHouseDeed ),
			typeof( ThatchedRoofCottageDeed ),
			typeof( BrickHouseDeed ),
			typeof( TwoStoryWoodPlasterHouseDeed ),
			typeof( TowerDeed ),
			typeof( KeepDeed ),
			typeof( CastleDeed ),
			typeof( LargePatioDeed ),
			typeof( LargeMarbleDeed ),
			typeof( SmallTowerDeed ),
			typeof( LogCabinDeed ),
			typeof( SandstonePatioDeed ),
			typeof( VillaDeed ),
			typeof( StoneWorkshopDeed ),
			typeof( MarbleWorkshopDeed ),
		};

		public ArchitectSellInfo()
		{
		}

		public int GetSellPriceFor( Item item )
		{
			int price = 0;
			
			if ( item is StonePlasterHouseDeed ) price = 43800;
			else if ( item is FieldStoneHouseDeed ) price = 43800;
			else if ( item is SmallBrickHouseDeed ) price = 43800;
			else if ( item is WoodHouseDeed ) price = 43800;
			else if ( item is WoodPlasterHouseDeed ) price = 43800;
			else if ( item is ThatchedRoofCottageDeed ) price = 43800;
			else if ( item is BrickHouseDeed ) price = 144500;
			else if ( item is TwoStoryWoodPlasterHouseDeed ) price = 192400;
			else if ( item is TowerDeed ) price = 433200;
			else if ( item is KeepDeed ) price = 665200;
			else if ( item is CastleDeed ) price = 1022800;
			else if ( item is LargePatioDeed ) price = 152800;
			else if ( item is LargeMarbleDeed ) price = 192800;
			else if ( item is SmallTowerDeed ) price = 88500;
			else if ( item is LogCabinDeed ) price = 97800;
			else if ( item is SandstonePatioDeed ) price = 90900;
			else if ( item is VillaDeed ) price = 136500;
			else if ( item is StoneWorkshopDeed ) price = 60600;
			else if ( item is MarbleWorkshopDeed ) price = 60300;
			else if ( item is SmallBrickHouseDeed ) price = 43800;
						
			return price;
		}

		public int GetBuyPriceFor( Item item )
		{
			return (int)( 1.00 * GetSellPriceFor( item ) );
		}

		public Type[] Types
		{
			get
			{
				return m_MyTypes;
			}
		}

		public string GetNameFor( Item item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return (1020000 + (item.ItemID & 0x3FFF)).ToString();
		}

		public bool IsSellable( Item item )
		{
			return IsInList( item.GetType() );
		}
 
		public bool IsResellable( Item item )
		{
			return IsInList( item.GetType() );
		}

		public bool IsInList( Type type )
		{
			for ( int i = 0; i < m_MyTypes.Length; i++ )
				if ( m_MyTypes[i] == type )
					return true;

			return false;
		}
	}
}
