using System;
using System.Collections;
using Server.Misc;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Sixth
{
	public class MassCurseSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Mass Curse", "Vas Des Sanct",
				SpellCircle.Sixth,
				Reagent.Garlic,
				Reagent.Nightshade,
				Reagent.MandrakeRoot,
				Reagent.SulfurousAsh
			);

		public MassCurseSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( IPoint3D p )
		{
			if ( !Caster.CanSee( p ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckSequence() )
			{
				if ( p is Item )
					p = ((Item)p).GetWorldTop();

				ArrayList targets = new ArrayList();

				IPooledEnumerable eable = Caster.Map.GetMobilesInRange( new Point3D( p ), 3 );

				foreach ( Mobile m in eable )
				{
					if ( Caster.CanSee( m ) && Caster.CanBeHarmful( m ) )
						targets.Add( m );
				}

				eable.Free();

				if ( Caster != p )
					Caster.Direction = Caster.GetDirectionTo( p );

				for ( int i = 0; i < targets.Count; ++i )
				{
					Mobile m = (Mobile)targets[i];

					Caster.DoHarmful( m );

					StatModHelper.AddStatCurse( Caster, m, StatType.Str );
					StatModHelper.AddStatCurse( Caster, m, StatType.Dex );
					StatModHelper.AddStatCurse( Caster, m, StatType.Int );

					m.PlaySound( 0x1E1 );
					Effects.SendTargetEffect( m, 0x374A, 16 );
				}
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private MassCurseSpell m_Owner;

			public InternalTarget( MassCurseSpell owner ) : base( 12, true, TargetFlags.None )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				IPoint3D p = o as IPoint3D;

				if ( p != null )
					m_Owner.Target( p );
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}